#include "StdAfx.h"
#include ".\abstractbutton.h"
#include "ImageProxy.h"
#include "RootContainer.h"
#include "FullFillLayoutManager.h"

LPCTSTR AbstractButtonProperty::Pushed = _T("pushed");

/************************************************************************/
/* IButtonUI */
/************************************************************************/
IMPLEMENT_RTTI(IButtonUI, IComponentUI)

/************************************************************************/
/* CAbstractButton */
/************************************************************************/
IMPLEMENT_RTTI(CAbstractButton, CNControl) 
CAbstractButton::CAbstractButton(void)
{
	m_bEnableToolTip = FALSE;
	m_bPushed = FALSE;
	m_eState = E_BUTTON_STATE_NONE;
	m_eStyle = E_BUTTON_STYLE_STAND;
	m_pBtnStateCgDg = &m_BtnStateCgDg_Def;
	m_bMouseDown = FALSE;

	m_ToolTip.SetOwner(this);
	// [:2008-12-18] Memory Leak  : SetLayoutManager AutoReleaseϱ ؼ Ʒ  
	//this->SetLayoutManager(new CFullFillLayoutManager());
	CFullFillLayoutManager* layoutManager = new CFullFillLayoutManager();
	this->SetLayoutManager(layoutManager);
	layoutManager->AutoRelease();

}

CAbstractButton::~CAbstractButton(void)
{
}
void CAbstractButton::SetStyle(eButtonStyle eStyle)
{
	m_eStyle = eStyle;
}

CAbstractButton::eButtonStyle CAbstractButton::GetStyle()
{
	return m_eStyle;
}

BOOL CAbstractButton::GetPushed()
{
	return m_bPushed;
}

void CAbstractButton::SetPushed(BOOL bPushed)
{
	if (bPushed != m_bPushed)
	{
		m_bPushed = bPushed;
		
		CPropertyChangeEvent PropEvent(this, AbstractButtonProperty::Pushed);
		this->FirePropertyChangedEvent(&PropEvent);
	}
}
CAbstractButton::eButtonState CAbstractButton::GetState()
{
	return m_eState;
}
void CAbstractButton::AddActionListener(IActionListener* pActionListener)
{
	ATLASSERT(pActionListener != NULL);
	m_vActionListener.AddTail(pActionListener);
	pActionListener->AddRef();
}
void CAbstractButton::RemoveActionListener(IActionListener* pActionListener)
{
	ATLASSERT(pActionListener != NULL);
	POSITION pos = m_vActionListener.Find(pActionListener);
	if (pos)
	{
		m_vActionListener.RemoveAt(pos);
		pActionListener->AutoRelease();
	}
}
// PM-896 2007/11/08 14:58:32 
/* {kr}
	۹ư  ,  콺 Ŭ 2ϸ,
	Ŭ νĵǾ OnLButtonDblClk  ߻ϰ OnLButtonDown  ߻ ʾ.
	ư Ŭ ʿ  OnLButtonDblClk  OnLButtonDown νϰ .
{/kr} */
void CAbstractButton::OnLButtonDblClk(UINT nFlags, CPoint point)
{
	this->OnLButtonDown(nFlags, point);
}
void CAbstractButton::OnLButtonDown(UINT nFlags, CPoint point) 
{
	__super::OnLButtonDown(nFlags, point);
	this->_OnLBtnDown();
	this->Invalidate();
}

void CAbstractButton::OnLButtonUp(UINT nFlags, CPoint point)
{
	__super::OnLButtonUp(nFlags, point);
	this->_OnLBtnUp();
	this->Invalidate();
}

void CAbstractButton::OnMouseMove(UINT nFlags, CPoint point)
{
	__super::OnMouseMove(nFlags, point);

	/** {kr} @n
	׹ȣ : ڹ @n
	 : 2008/07/01 @n
	۾ : hhd2002 @n
	 :  Ŭ  @n
	 : @n
		 1. ư-down @n
		 2. ư 콺  @n
		 3. ư ̹ ư-over ̹ . @n
	ذ : @n
		-----------------------------------------------------------------------------	@n
		mouse-down-flag	capture		mouse-in	Ȳ			ó					@n
		-----------------------------------------------------------------------------	@n
				T			T			T		Ŭ Ȳ		UI=selected			@n
		.............................................................................	@n
				T			T			F		Ŭǰ 	UI=over				@n
																tooltip-hide			@n
		.............................................................................	@n
				T			F			T		Ұ			mouse-down-flag=F,		@n
																UI=over				@n
		.............................................................................	@n
				T			F			F		Ұ			mouse-down-flag=F,		@n
																UI=normal			@n
																tooltip-hide			@n
		.............................................................................	@n
				F			T			T		Ұ			release-capture,		@n
																UI=over				@n
		.............................................................................	@n
				F			T			F		Ұ			release-capture,		@n
																UI=normal			@n
																tooltip-hide			@n
		.............................................................................	@n
				F			F			T		׳ɸ콺		UI=over				@n
																					@n
		.............................................................................	@n
				F			F			F		Ұ			UI=normal			@n
																tooltip-hide			@n
		-----------------------------------------------------------------------------	@n
	{/kr} */	

	if (m_bMouseDown == TRUE)
	{
		if (this->GetRootContainer()->GetCapturedComponent() == this)
		{
			if (this->IsMouseOver() == TRUE)
			{
				m_eState = E_BUTTON_STATE_SELECTED;
			}
			else
			{
				m_eState = E_BUTTON_STATE_OVER;
				m_ToolTip.SetVisible(FALSE);
			}
		}
		else
		{
			if (this->IsMouseOver() == TRUE)
			{
				m_bMouseDown = FALSE;
				m_eState = E_BUTTON_STATE_OVER;
			}
			else
			{
				m_bMouseDown = FALSE;
				m_eState = E_BUTTON_STATE_NONE;
				m_ToolTip.SetVisible(FALSE);
			}
		}
	}
	else
	{
		if (this->GetRootContainer()->GetCapturedComponent() == this)
		{
			if (this->IsMouseOver() == TRUE)
			{
				this->ReleaseCapture();
				m_eState = E_BUTTON_STATE_OVER;
			}
			else
			{
				this->ReleaseCapture();
				m_eState = E_BUTTON_STATE_NONE;
				m_ToolTip.SetVisible(FALSE);
			}
		}
		else
		{
			if (this->IsMouseOver() == TRUE)
			{
				m_eState = E_BUTTON_STATE_OVER;
			}
			else
			{
				m_eState = E_BUTTON_STATE_NONE;
				m_ToolTip.SetVisible(FALSE);
			}
		}
	}

	this->Invalidate();
}
void CAbstractButton::OnMouseEnter(CPoint point)
{
	__super::OnMouseEnter(point);
	m_ToolTip.SetVisible(TRUE);
}
void CAbstractButton::OnMouseLeave()
{
	__super::OnMouseLeave();
	m_ToolTip.SetVisible(FALSE);
	m_eState = E_BUTTON_STATE_NONE;
	this->Invalidate();
}
void CAbstractButton::FireAction()
{
	if(this->IsWindowEnabled() == FALSE) return;

	POSITION pos = m_vActionListener.GetHeadPosition();
	while (pos)
	{
		IActionListener* pActionListener = m_vActionListener.GetNext(pos);
		if (pActionListener)
		{
			pActionListener->OnActionPerformed(reinterpret_cast<CComponent*>(this));
		}
	}
}
void CAbstractButton::SetToolTipText(LPCTSTR szToolTipText)
{
	// {kr} Է° ȿ ˻ {/kr}
	ATLASSERT(_tcslen(szToolTipText) >= 0);
	if(!(_tcslen(szToolTipText) >= 0)) return;

	m_ToolTip.SetText(szToolTipText);
}

void CAbstractButton::SetBtnStateBgDg( IButtonStateChangeDelegate* pBtnStateBgDg )
{
	ATLASSERT(pBtnStateBgDg);
	m_pBtnStateCgDg = pBtnStateBgDg;
}

// PM-690 2007/10/19 12:23:56 
/* {kr}
	ư  콺 ̵ÿ Ʈ Ǿµ.
	ư show ɶ  Ʈ ǵ .
{/kr} */
void CAbstractButton::OnShowNotify()
{
	HWND hwndOwner = this->GetOwnerWindow();
	if(::IsWindow(hwndOwner) == TRUE)
	{
		DWORD dwThis = DWORD(this);
		::PostMessage(hwndOwner, UM_INVOKELATER_SHOWTOOLTIP, WPARAM(dwThis), NULL);
	}
	__super::OnShowNotify();
}

void CAbstractButton::OnHideNotify()
{
	// {kr}   {/kr}
	m_ToolTip.SetVisible(FALSE);
	__super::OnHideNotify();
}

void CAbstractButton::DoClick()
{
	this->_OnLBtnDown();
	this->_OnLBtnUp();
	this->Invalidate();
}
void CAbstractButton::_OnLBtnDown()
{
	m_eState = E_BUTTON_STATE_SELECTED;
	this->SetFocus();
	this->SetCapture();
	m_bMouseDown = TRUE;
}
void CAbstractButton::_OnLBtnUp()
{
	// PM-717 : OnLButtonDown ȣǾ  FireAction ϵ Ѵ.
	if (m_bMouseDown == TRUE &&
		this->IsMouseOver() == TRUE)
	{
		if( m_eStyle == E_BUTTON_STYLE_PUSHED )
		{
			ATLASSERT(m_pBtnStateCgDg);
			if (m_pBtnStateCgDg->IsChangingAvailable(this) == TRUE)
			{
				// {kr} 氡ϴٸ {/kr}
				m_bPushed = !m_bPushed;
				m_eState = E_BUTTON_STATE_OVER;
				FireAction();
				CPropertyChangeEvent PropEvent(this, AbstractButtonProperty::Pushed);
				this->FirePropertyChangedEvent(&PropEvent);
			}
		}
		else
		{
			m_eState = E_BUTTON_STATE_OVER;
			FireAction();
		}
	}

	m_bMouseDown = FALSE;
	this->ReleaseCapture();
}
void CAbstractButton::OnSize(INT32 nWidth, INT32 nHeight)
{
	__super::OnSize(nWidth, nHeight);

	// {kr} Ʈ ưƮ  ʵ  . {/kr}
	CPoint ptToolTip;
	ptToolTip.SetPoint(		
		Prop_ToolTipOffsetX,
		Prop_ToolTipOffsetY + nHeight);
	m_ToolTip.SetPosition(&ptToolTip);
}

LRESULT CAbstractButton::_OnInvokeLaterShowToolTip( UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& bHandled )
{
	if (DWORD(wParam) != DWORD(this))
	{
		bHandled = FALSE;
		return S_FALSE;
	}

	// {kr} Ʈ  콺 Ͱ ÷ִ Ǻ  visible . {/kr}
	BOOL bIsToolTipVisible = this->IsMouseOver();
	m_ToolTip.SetVisible(bIsToolTipVisible);
	
	return S_OK;
}

BOOL CAbstractButton::EnableWindow(BOOL bEnabled)
{
	// DisableǾ  MouseCaptureǾ ReleaseCapture ʾƼ 
	// 콺 ̺Ʈ ȵ  ߻ؼ EnableWindow ̵ 
	//  Ʈ ܿ óؾ 
	BOOL bRet = __super::EnableWindow(bEnabled);
	if (bEnabled == FALSE)
		_OnLBtnUp();
	return bRet;
}